==> radiusclient-0.1.tar.gz: radiusclient-0.1/src/radius.h <==
/*
 * $Id: radius.h,v 1.7 1996/01/06 01:58:18 lf Exp $
 *
 * Copyright (C) 1995 Lars Fenneberg
 *
 * Copyright 1992 Livingston Enterprises, Inc.
 *
 * Copyright 1992,1993, 1994,1995 The Regents of the University of Michigan 
 * and Merit Network, Inc. All Rights Reserved
 *
 * See the file COPYRIGHT for the respective terms and conditions. 
 * If the file is missing contact me at in5y050@public.uni-hamburg.de 
 * and I'll send you a copy.
 *
 */

#ifndef RADIUS_H
#define RADIUS_H

#include        <sys/socket.h>
#include        <netinet/in.h>
#include	<stdio.h>
#include	<time.h>

typedef unsigned long UINT4;
typedef long 	      INT4;

#define AUTH_VECTOR_LEN		16
#define AUTH_PASS_LEN		16
#define AUTH_ID_LEN		64
#define AUTH_STRING_LEN		128	/* maximum of 253 */

#define FILE_PFX_LEN		16
#define FILTER_LEN		16
#define NAME_LENGTH		32

/* codes for radius_buildreq, radius_getport, etc. */
#define AUTH			0
#define ACCT			1

/* defines for config.c */

#define SERVER_MAX 10
#define AUTH_LOCAL_FST	(1<<0)
#define AUTH_RADIUS_FST (1<<1)
#define AUTH_LOCAL_SND  (1<<2)
#define AUTH_RADIUS_SND (1<<3)

typedef struct pw_auth_hdr
{
	u_char          code;
	u_char          id;
	u_short         length;
	u_char          vector[AUTH_VECTOR_LEN];
	u_char          data[2];
} AUTH_HDR;

#define AUTH_HDR_LEN			20
#define MAX_SECRET_LENGTH		16
#define CHAP_VALUE_LENGTH		16

#define PW_AUTH_UDP_PORT		1645
#define PW_ACCT_UDP_PORT		1646

#define PW_TYPE_STRING			0
#define PW_TYPE_INTEGER			1
#define PW_TYPE_IPADDR			2
#define PW_TYPE_DATE			3

/* standard RADIUS codes */

#define	PW_ACCESS_REQUEST		1
#define	PW_ACCESS_ACCEPT		2
#define	PW_ACCESS_REJECT		3
#define	PW_ACCOUNTING_REQUEST		4
#define	PW_ACCOUNTING_RESPONSE		5
#define	PW_ACCOUNTING_STATUS		6
#define	PW_PASSWORD_REQUEST		7
#define	PW_PASSWORD_ACK			8
#define	PW_PASSWORD_REJECT		9
#define	PW_ACCOUNTING_MESSAGE		10
#define	PW_ACCESS_CHALLENGE		11
#define	PW_STATUS_SERVER		12
#define	PW_STATUS_CLIENT		13


/* standard RADIUS attribute-value pairs */

#define	PW_USER_NAME			1	/* string */
#define	PW_USER_PASSWORD		2	/* string */
#define	PW_CHAP_PASSWORD		3	/* string */
#define	PW_NAS_IP_ADDRESS		4	/* ipaddr */
#define	PW_NAS_PORT			5	/* integer */
#define	PW_SERVICE_TYPE			6	/* integer */
#define	PW_FRAMED_PROTOCOL		7	/* integer */
#define	PW_FRAMED_IP_ADDRESS		8	/* ipaddr */
#define	PW_FRAMED_IP_NETMASK		9	/* ipaddr */
#define	PW_FRAMED_ROUTING		10	/* integer */
#define	PW_FILTER_ID		        11	/* string */
#define	PW_FRAMED_MTU			12	/* integer */
#define	PW_FRAMED_COMPRESSION		13	/* integer */
#define	PW_LOGIN_IP_HOST		14	/* ipaddr */
#define	PW_LOGIN_SERVICE		15	/* integer */
#define	PW_LOGIN_PORT			16	/* integer */
#define	PW_OLD_PASSWORD			17	/* string */ /* deprecated */
#define	PW_REPLY_MESSAGE		18	/* string */
#define	PW_LOGIN_CALLBACK_NUMBER	19	/* string */
#define	PW_FRAMED_CALLBACK_ID		20	/* string */
#define	PW_EXPIRATION			21	/* date */ /* deprecated */
#define	PW_FRAMED_ROUTE			22	/* string */
#define	PW_FRAMED_IPX_NETWORK		23	/* integer */
#define	PW_STATE			24	/* string */
#define	PW_CLASS			25	/* string */
#define	PW_VENDOR_SPECIFIC		26	/* string */
#define	PW_SESSION_TIMEOUT		27	/* integer */
#define	PW_IDLE_TIMEOUT			28	/* integer */
#define	PW_TERMINATION_ACTION		29	/* integer */
#define	PW_CLIENT_PORT_DNIS		30	/* string */
#define	PW_CALLER_ID			31	/* string */
#define	PW_NAS_IDENTIFIER		32	/* string */
#define	PW_PROXY_STATE			33	/* string */
#define	PW_LOGIN_LAT_SERVICE		34	/* string */
#define	PW_LOGIN_LAT_NODE		35	/* string */
#define	PW_LOGIN_LAT_GROUP		36	/* string */
#define	PW_FRAMED_APPLETALK_LINK	37	/* integer */
#define	PW_FRAMED_APPLETALK_NETWORK	38	/* integer */
#define	PW_FRAMED_APPLETALK_ZONE	39	/* string */

/*	Accounting */

#define	PW_ACCT_STATUS_TYPE		40	/* integer */
#define	PW_ACCT_DELAY_TIME		41	/* integer */
#define	PW_ACCT_INPUT_OCTETS		42	/* integer */ /* not impl. */
#define	PW_ACCT_OUTPUT_OCTETS		43	/* integer */ /* not impl. */
#define	PW_ACCT_SESSION_ID		44	/* string */
#define	PW_ACCT_AUTHENTIC		45	/* integer */
#define	PW_ACCT_SESSION_TIME		46	/* integer */
#define	PW_ACCT_INPUT_PACKETS		47	/* integer */ /* not impl. */
#define	PW_ACCT_OUTPUT_PACKETS		48	/* integer */ /* not impl. */

/*	Merit Experimental Extensions */

#define	PW_INFO_PORT			210	/* integer */
#define	PW_PROXY_ACTION			211	/* string */
#define	PW_SIGNATURE			212	/* string */
#define	PW_TOKEN			213	/* string */
#define	PW_ACCT_RATE			214	/* string */
#define	PW_ACCT_CHARGE			215	/* string */
#define	PW_ACCT_TRANSACTION_ID		216	/* string */
#define	PW_ACCT_CHARGE_ALLOWED		217	/* string */
#define	PW_MAXIMUM_TIME			218	/* integer */
#define	PW_AVAILABLE_TIME		219	/* integer */
#define	PW_TIME_USED			220	/* integer */
#define	PW_HUNTGROUP_NAME		221	/* string */
#define	PW_USER_ID			222	/* string */
#define	PW_USER_REALM			223	/* string */

/*	Integer Translations */

/*	SERVICE TYPES	*/

#define	PW_LOGIN			1
#define	PW_FRAMED			2
#define	PW_CALLBACK_LOGIN		3
#define	PW_CALLBACK_FRAMED		4
#define	PW_OUTBOUND_USER		5
#define	PW_ADMINISTRATIVE_USER		6
#define	PW_EXEC_USER			7
#define PW_AUTHENTICATE_ONLY		8

/*	FRAMED PROTOCOLS	*/

#define	PW_PPP				1
#define	PW_SLIP				2

/*	FRAMED ROUTING VALUES	*/

#define	PW_NONE				0
#define	PW_BROADCAST			1
#define	PW_LISTEN			2
#define	PW_BROADCAST_LISTEN		3

/*	FRAMED COMPRESSION TYPES	*/

#define	PW_VAN_JACOBSON_TCP_IP		1
#define	PW_IPX_HEADER_COMPRESSION	2

/*	LOGIN SERVICES	*/

#define	PW_TELNET			0
#define	PW_RLOGIN			1
#define	PW_TCP_CLEAR			2
#define	PW_PORTMASTER			3
#define	PW_LAT				4

/*	TERMINATION ACTIONS	*/

#define	PW_DEFAULT			0
#define	PW_RADIUS_REQUEST		1

/*	PROHIBIT PROTOCOL  */

#define PW_DUMB		0	/* 1 and 2 are defined in FRAMED PROTOCOLS */
#define PW_AUTH_ONLY	3
#define PW_ALL		255

/*	ACCOUNTING STATUS TYPES    */

#define PW_STATUS_START		1
#define PW_STATUS_STOP		2
#define PW_STATUS_ALIVE		3
#define PW_STATUS_MODEM_START	4
#define PW_STATUS_MODEM_STOP	5
#define PW_STATUS_CANCEL	6

/* Default Configuration File Name */
/* normally set in Makefile */

#ifndef RADIUSCLIENT_CONF
#define RADIUSCLIENT_CONF	"/etc/radiusclient.conf"
#endif

/* Server data structures */

typedef struct dict_attr
{
	char              name[NAME_LENGTH + 1];	/* attribute name */
	int               value;			/* attribute index */
	int               type;				/* string, int, etc. */
	struct dict_attr *next;
} DICT_ATTR;

typedef struct dict_value
{
	char               attrname[NAME_LENGTH +1];
	char               name[NAME_LENGTH + 1];
	int                value;
	struct dict_value *next;
} DICT_VALUE;

typedef struct value_pair
{
	char               name[NAME_LENGTH + 1];
	int                attribute;
	int                type;
	UINT4              lvalue;
	char               strvalue[AUTH_STRING_LEN + 1];
	struct value_pair *next;
} VALUE_PAIR;

typedef struct auth_req
{
	UINT4             ipaddr;	    /* IP address of requestor */
	u_short           udp_port;         /* UDP reply socket of requestor */
	u_char            id;               /* Original request seq. number */
	u_char            code;             /* Type of RADIUS packet */
	u_char            vector[AUTH_VECTOR_LEN];
	u_char            secret[MAX_SECRET_LENGTH + 1];
	u_char            ttl;              /* Global queue time-to-live secs */
	u_char            timer;            /* General utility timer */
	u_char            reply_id;         /* RADIUS-to-RADIUS seq. number */
	u_char            retry_cnt;        /* Counter for duplicate requests */
	u_char            file_pfx[FILE_PFX_LEN + 1];
	u_char            realm_filter[FILTER_LEN + 1];
	u_char            state;            /* State of current request */
	int               result;           /* Result of previous action */
	int               cur_count;        /* Original number request pairs */
	struct aatv      *fsm_aatv;         /* Pointer to current FSM action */
	struct aatv      *direct_aatv;      /* Pointer to actual action */
	struct event_ent *event_q;          /* Pointer to active event queue */
	struct auth_req  *next;             /* Global request queue link */
	VALUE_PAIR       *request;          /* Original client a/v pairs */
	VALUE_PAIR       *cur_request;      /* Represents current a/v pairs */
	VALUE_PAIR       *user_check;       /* List of users file check items */
	VALUE_PAIR       *user_reply;       /* List of users file reply items */
} AUTH_REQ;


#define MGMT_POLL_SECRET	"Hardlyasecret"

/* 	Define return codes from "SendServer" utility */

#define BADRESP_RC	-2
#define ERROR_RC	-1
#define OK_RC		0
#define TIMEOUT_RC	1

typedef struct send_data /* Used to pass information to sendserver() function */
{
	u_char          code;		/* RADIUS packet code */
	u_char          seq_nbr;	/* Packet sequence number */
	char           *server;		/* Name/addrress of RADIUS server */
	int             svc_port;	/* RADIUS protocol destination port */
	int             timeout;	/* Session timeout in seconds */
	int		retries;
	VALUE_PAIR     *send_pairs;     /* More a/v pairs to send */
	VALUE_PAIR     *receive_pairs;  /* Where to place received a/v pairs */
} SEND_DATA;

#define MIN(a, b)     ((a) < (b) ? (a) : (b))
#define MAX(a, b)     ((a) > (b) ? (a) : (b))


/*	Function prototypes	*/

/*	avpair.c		*/

VALUE_PAIR *rc_avpair_add (VALUE_PAIR **, int, void *);
int rc_avpair_assign (VALUE_PAIR *, void *);
VALUE_PAIR *rc_avpair_new (int, void *);
VALUE_PAIR *rc_avpair_gen (AUTH_HDR *);
VALUE_PAIR *rc_avpair_get (VALUE_PAIR *, UINT4);
void rc_avpair_insert (VALUE_PAIR **, VALUE_PAIR *, VALUE_PAIR *);
void rc_avpair_free (VALUE_PAIR *);
int rc_avpair_parse (char *, VALUE_PAIR **);
void rc_avpair_tostr (VALUE_PAIR *, char *, int, char *, int);
VALUE_PAIR *rc_avpair_readin(FILE *);

/*	buildreq.c		*/

void rc_buildreq(SEND_DATA *, int, char *, unsigned short, int, int);
unsigned char rc_get_seqnbr(void);
int rc_auth(UINT4, VALUE_PAIR *, VALUE_PAIR **, char *);
int rc_acct(UINT4, VALUE_PAIR *);
int rc_check(char *, unsigned short);

/*	clientid.c		*/

int rc_read_mapfile(char *);
UINT4 rc_map2id(char *);

/*	config.c		*/

int rc_read_config(char *);
int rc_find_server (char *, UINT4 *, char *);

/*	dict.c			*/

int rc_read_dictionary (char *);
DICT_ATTR *rc_dict_getattr (int);
DICT_ATTR *rc_dict_findattr (char *);
DICT_VALUE *rc_dict_findval (char *);
DICT_VALUE * rc_dict_getval (UINT4, char *);

/*	ip_util.c		*/

UINT4 rc_get_ipaddr (char *);
int rc_good_ipaddr (char *);
char *rc_ip_hostname (UINT4);
unsigned short rc_getport(int);
UINT4 rc_own_ipaddress(void);


/*	log.c			*/

void rc_log(int, char *, ...);

/*	sendserver.c		*/

int rc_send_server (SEND_DATA *, char *);

/*	util.c			*/

void rc_str2tm (char *, struct tm *);
char *rc_mksid(void);
char *rc_getifname(char *);
char *rc_getstr (char *, int);

/*	external declarations	*/

extern char *conf_authserver[SERVER_MAX];
extern char *conf_acctserver[SERVER_MAX];
extern int  conf_authserver_port[SERVER_MAX];
extern int  conf_acctserver_port[SERVER_MAX];
extern char *conf_clients;
extern char *conf_dictionary;
extern char *conf_default_realm;
extern int conf_auth_type;
extern int conf_radius_timeout;
extern int conf_radius_retries;
extern char *conf_login_local;
extern char *conf_login_radius;
extern int conf_login_tries;
extern char *conf_seqfile;
extern int conf_login_timeout;
extern char *conf_mapfile;

extern int conf_numauth;	/* number of authentication servers */
extern int conf_numacct;	/* number of accounting servers */

#endif /* RADIUS_H */

==> radiusclient-0.2.1.tar.gz: radiusclient-0.2.1/src/radius.h <==
/*
 * $Id: radius.h,v 1.8 1996/03/15 13:53:04 lf Exp $
 *
 * Copyright (C) 1995, 1996 Lars Fenneberg
 *
 * Copyright 1992 Livingston Enterprises, Inc.
 *
 * Copyright 1992,1993, 1994,1995 The Regents of the University of Michigan 
 * and Merit Network, Inc. All Rights Reserved
 *
 * See the file COPYRIGHT for the respective terms and conditions. 
 * If the file is missing contact me at in5y050@public.uni-hamburg.de 
 * and I'll send you a copy.
 *
 */

#ifndef RADIUS_H
#define RADIUS_H

#include	<sys/types.h>
#include        <sys/socket.h>
#include        <netinet/in.h>
#include	<stdio.h>
#include	<time.h>

typedef unsigned long UINT4;
typedef long 	      INT4;

#define AUTH_VECTOR_LEN		16
#define AUTH_PASS_LEN		16
#define AUTH_ID_LEN		64
#define AUTH_STRING_LEN		128	/* maximum of 253 */

#define FILE_PFX_LEN		16
#define FILTER_LEN		16
#define NAME_LENGTH		32

/* codes for radius_buildreq, radius_getport, etc. */
#define AUTH			0
#define ACCT			1

/* defines for config.c */

#define SERVER_MAX 10
#define AUTH_LOCAL_FST	(1<<0)
#define AUTH_RADIUS_FST (1<<1)
#define AUTH_LOCAL_SND  (1<<2)
#define AUTH_RADIUS_SND (1<<3)

typedef struct pw_auth_hdr
{
	u_char          code;
	u_char          id;
	u_short         length;
	u_char          vector[AUTH_VECTOR_LEN];
	u_char          data[2];
} AUTH_HDR;

#define AUTH_HDR_LEN			20
#define MAX_SECRET_LENGTH		16
#define CHAP_VALUE_LENGTH		16

#define PW_AUTH_UDP_PORT		1645
#define PW_ACCT_UDP_PORT		1646

#define PW_TYPE_STRING			0
#define PW_TYPE_INTEGER			1
#define PW_TYPE_IPADDR			2
#define PW_TYPE_DATE			3

/* standard RADIUS codes */

#define	PW_ACCESS_REQUEST		1
#define	PW_ACCESS_ACCEPT		2
#define	PW_ACCESS_REJECT		3
#define	PW_ACCOUNTING_REQUEST		4
#define	PW_ACCOUNTING_RESPONSE		5
#define	PW_ACCOUNTING_STATUS		6
#define	PW_PASSWORD_REQUEST		7
#define	PW_PASSWORD_ACK			8
#define	PW_PASSWORD_REJECT		9
#define	PW_ACCOUNTING_MESSAGE		10
#define	PW_ACCESS_CHALLENGE		11
#define	PW_STATUS_SERVER		12
#define	PW_STATUS_CLIENT		13


/* standard RADIUS attribute-value pairs */

#define	PW_USER_NAME			1	/* string */
#define	PW_USER_PASSWORD		2	/* string */
#define	PW_CHAP_PASSWORD		3	/* string */
#define	PW_NAS_IP_ADDRESS		4	/* ipaddr */
#define	PW_NAS_PORT			5	/* integer */
#define	PW_SERVICE_TYPE			6	/* integer */
#define	PW_FRAMED_PROTOCOL		7	/* integer */
#define	PW_FRAMED_IP_ADDRESS		8	/* ipaddr */
#define	PW_FRAMED_IP_NETMASK		9	/* ipaddr */
#define	PW_FRAMED_ROUTING		10	/* integer */
#define	PW_FILTER_ID		        11	/* string */
#define	PW_FRAMED_MTU			12	/* integer */
#define	PW_FRAMED_COMPRESSION		13	/* integer */
#define	PW_LOGIN_IP_HOST		14	/* ipaddr */
#define	PW_LOGIN_SERVICE		15	/* integer */
#define	PW_LOGIN_PORT			16	/* integer */
#define	PW_OLD_PASSWORD			17	/* string */ /* deprecated */
#define	PW_REPLY_MESSAGE		18	/* string */
#define	PW_LOGIN_CALLBACK_NUMBER	19	/* string */
#define	PW_FRAMED_CALLBACK_ID		20	/* string */
#define	PW_EXPIRATION			21	/* date */ /* deprecated */
#define	PW_FRAMED_ROUTE			22	/* string */
#define	PW_FRAMED_IPX_NETWORK		23	/* integer */
#define	PW_STATE			24	/* string */
#define	PW_CLASS			25	/* string */
#define	PW_VENDOR_SPECIFIC		26	/* string */
#define	PW_SESSION_TIMEOUT		27	/* integer */
#define	PW_IDLE_TIMEOUT			28	/* integer */
#define	PW_TERMINATION_ACTION		29	/* integer */
#define	PW_CALLED_STATION_ID            30      /* string */
#define	PW_CALLING_STATION_ID           31      /* string */
#define	PW_NAS_IDENTIFIER		32	/* string */
#define	PW_PROXY_STATE			33	/* string */
#define	PW_LOGIN_LAT_SERVICE		34	/* string */
#define	PW_LOGIN_LAT_NODE		35	/* string */
#define	PW_LOGIN_LAT_GROUP		36	/* string */
#define	PW_FRAMED_APPLETALK_LINK	37	/* integer */
#define	PW_FRAMED_APPLETALK_NETWORK	38	/* integer */
#define	PW_FRAMED_APPLETALK_ZONE	39	/* string */
#define	PW_CHAP_CHALLENGE               60      /* string */
#define	PW_NAS_PORT_TYPE                61      /* integer */
#define	PW_PORT_LIMIT                   62      /* integer */


/*	Accounting */

#define	PW_ACCT_STATUS_TYPE		40	/* integer */
#define	PW_ACCT_DELAY_TIME		41	/* integer */
#define	PW_ACCT_INPUT_OCTETS		42	/* integer */ /* not impl. */
#define	PW_ACCT_OUTPUT_OCTETS		43	/* integer */ /* not impl. */
#define	PW_ACCT_SESSION_ID		44	/* string */
#define	PW_ACCT_AUTHENTIC		45	/* integer */
#define	PW_ACCT_SESSION_TIME		46	/* integer */
#define	PW_ACCT_INPUT_PACKETS		47	/* integer */ /* not impl. */
#define	PW_ACCT_OUTPUT_PACKETS		48	/* integer */ /* not impl. */

/*	Merit Experimental Extensions */

#define PW_AVAILABLE_TIME               209     /* integer */
#define	PW_INFO_PORT			210	/* integer */
#define	PW_PROXY_ACTION			211	/* string */
#define	PW_SIGNATURE			212	/* string */
#define	PW_TOKEN			213	/* string */
#define	PW_ACCT_RATE			214	/* string */
#define	PW_ACCT_CHARGE			215	/* string */
#define	PW_ACCT_TRANSACTION_ID		216	/* string */
#define	PW_ACCT_CHARGE_ALLOWED		217	/* string */
#define	PW_MAXIMUM_TIME			218	/* integer */
/* #define     unavailable             219         ??????? */
#define	PW_TIME_USED			220	/* integer */
#define	PW_HUNTGROUP_NAME		221	/* string */
#define	PW_USER_ID			222	/* string */
#define	PW_USER_REALM			223	/* string */

/*	Integer Translations */

/*	SERVICE TYPES	*/

#define	PW_LOGIN			1
#define	PW_FRAMED			2
#define	PW_CALLBACK_LOGIN		3
#define	PW_CALLBACK_FRAMED		4
#define	PW_OUTBOUND_USER		5
#define	PW_ADMINISTRATIVE_USER		6
#define PW_SHELL_USER                   7
#define PW_AUTHENTICATE_ONLY		8
#define PW_CALLBACK_ADMIN_USER         9

/*	FRAMED PROTOCOLS	*/

#define	PW_PPP				1
#define	PW_SLIP				2

/*	FRAMED ROUTING VALUES	*/

#define	PW_NONE				0
#define	PW_BROADCAST			1
#define	PW_LISTEN			2
#define	PW_BROADCAST_LISTEN		3

/*	FRAMED COMPRESSION TYPES	*/

#define	PW_VAN_JACOBSON_TCP_IP		1
#define	PW_IPX_HEADER_COMPRESSION	2

/*	LOGIN SERVICES	*/

#define	PW_TELNET			0
#define	PW_RLOGIN			1
#define	PW_TCP_CLEAR			2
#define	PW_PORTMASTER			3
#define	PW_LAT				4

/*	TERMINATION ACTIONS	*/

#define	PW_DEFAULT			0
#define	PW_RADIUS_REQUEST		1

/*	PROHIBIT PROTOCOL  */

#define PW_DUMB		0	/* 1 and 2 are defined in FRAMED PROTOCOLS */
#define PW_AUTH_ONLY	3
#define PW_ALL		255

/*	ACCOUNTING STATUS TYPES    */

#define PW_STATUS_START		1
#define PW_STATUS_STOP		2
#define PW_STATUS_ALIVE		3
#define PW_STATUS_MODEM_START	4
#define PW_STATUS_MODEM_STOP	5
#define PW_STATUS_CANCEL	6
#define PW_ACCOUNTING_ON	7
#define PW_ACCOUNTING_OFF	8
 
/*     NAS PORT TYPES    */

#define PW_ASYNC		0
#define PW_SYNC			1
#define PW_ISDN_SYNC		2
#define PW_ISDN_SYNC_V120	3
#define PW_ISDN_SYNC_V110	4

/* Default Configuration File Name */
/* normally set in Makefile */

#ifndef RADIUSCLIENT_CONF
#define RADIUSCLIENT_CONF	"/etc/radiusclient.conf"
#endif

/* Server data structures */

typedef struct dict_attr
{
	char              name[NAME_LENGTH + 1];	/* attribute name */
	int               value;			/* attribute index */
	int               type;				/* string, int, etc. */
	struct dict_attr *next;
} DICT_ATTR;

typedef struct dict_value
{
	char               attrname[NAME_LENGTH +1];
	char               name[NAME_LENGTH + 1];
	int                value;
	struct dict_value *next;
} DICT_VALUE;

typedef struct value_pair
{
	char               name[NAME_LENGTH + 1];
	int                attribute;
	int                type;
	UINT4              lvalue;
	char               strvalue[AUTH_STRING_LEN + 1];
	struct value_pair *next;
} VALUE_PAIR;

typedef struct auth_req
{
	UINT4             ipaddr;	    /* IP address of requestor */
	u_short           udp_port;         /* UDP reply socket of requestor */
	u_char            id;               /* Original request seq. number */
	u_char            code;             /* Type of RADIUS packet */
	u_char            vector[AUTH_VECTOR_LEN];
	u_char            secret[MAX_SECRET_LENGTH + 1];
	u_char            ttl;              /* Global queue time-to-live secs */
	u_char            timer;            /* General utility timer */
	u_char            reply_id;         /* RADIUS-to-RADIUS seq. number */
	u_char            retry_cnt;        /* Counter for duplicate requests */
	u_char            file_pfx[FILE_PFX_LEN + 1];
	u_char            realm_filter[FILTER_LEN + 1];
	u_char            state;            /* State of current request */
	int               result;           /* Result of previous action */
	int               cur_count;        /* Original number request pairs */
	struct aatv      *fsm_aatv;         /* Pointer to current FSM action */
	struct aatv      *direct_aatv;      /* Pointer to actual action */
	struct event_ent *event_q;          /* Pointer to active event queue */
	struct auth_req  *next;             /* Global request queue link */
	VALUE_PAIR       *request;          /* Original client a/v pairs */
	VALUE_PAIR       *cur_request;      /* Represents current a/v pairs */
	VALUE_PAIR       *user_check;       /* List of users file check items */
	VALUE_PAIR       *user_reply;       /* List of users file reply items */
} AUTH_REQ;


#define MGMT_POLL_SECRET	"Hardlyasecret"

/* 	Define return codes from "SendServer" utility */

#define BADRESP_RC	-2
#define ERROR_RC	-1
#define OK_RC		0
#define TIMEOUT_RC	1

typedef struct send_data /* Used to pass information to sendserver() function */
{
	u_char          code;		/* RADIUS packet code */
	u_char          seq_nbr;	/* Packet sequence number */
	char           *server;		/* Name/addrress of RADIUS server */
	int             svc_port;	/* RADIUS protocol destination port */
	int             timeout;	/* Session timeout in seconds */
	int		retries;
	VALUE_PAIR     *send_pairs;     /* More a/v pairs to send */
	VALUE_PAIR     *receive_pairs;  /* Where to place received a/v pairs */
} SEND_DATA;

#define MIN(a, b)     ((a) < (b) ? (a) : (b))
#define MAX(a, b)     ((a) > (b) ? (a) : (b))

struct env
{
	int maxsize, size;
	char **env;
};

#define ENV_SIZE	128

/*	Function prototypes	*/

/*	avpair.c		*/

VALUE_PAIR *rc_avpair_add (VALUE_PAIR **, int, void *);
int rc_avpair_assign (VALUE_PAIR *, void *);
VALUE_PAIR *rc_avpair_new (int, void *);
VALUE_PAIR *rc_avpair_gen (AUTH_HDR *);
VALUE_PAIR *rc_avpair_get (VALUE_PAIR *, UINT4);
void rc_avpair_insert (VALUE_PAIR **, VALUE_PAIR *, VALUE_PAIR *);
void rc_avpair_free (VALUE_PAIR *);
int rc_avpair_parse (char *, VALUE_PAIR **);
void rc_avpair_tostr (VALUE_PAIR *, char *, int, char *, int);
VALUE_PAIR *rc_avpair_readin(FILE *);

/*	buildreq.c		*/

void rc_buildreq(SEND_DATA *, int, char *, unsigned short, int, int);
unsigned char rc_get_seqnbr(void);
int rc_auth(UINT4, VALUE_PAIR *, VALUE_PAIR **, char *);
int rc_acct(UINT4, VALUE_PAIR *);
int rc_check(char *, unsigned short);

/*	clientid.c		*/

int rc_read_mapfile(char *);
UINT4 rc_map2id(char *);

/*	config.c		*/

int rc_read_config(char *);
int rc_find_server (char *, UINT4 *, char *);

/*	dict.c			*/

int rc_read_dictionary (char *);
DICT_ATTR *rc_dict_getattr (int);
DICT_ATTR *rc_dict_findattr (char *);
DICT_VALUE *rc_dict_findval (char *);
DICT_VALUE * rc_dict_getval (UINT4, char *);

/*	ip_util.c		*/

UINT4 rc_get_ipaddr (char *);
int rc_good_ipaddr (char *);
char *rc_ip_hostname (UINT4);
unsigned short rc_getport(int);
UINT4 rc_own_ipaddress(void);


/*	log.c			*/

void rc_log(int, char *, ...);

/*	sendserver.c		*/

int rc_send_server (SEND_DATA *, char *);

/*	util.c			*/

void rc_str2tm (char *, struct tm *);
char *rc_mksid(void);
char *rc_getifname(char *);
char *rc_getstr (char *, int);

/*	env.c			*/

struct env *rc_new_env(int);
void rc_free_env(struct env *);
int rc_add_env(struct env *, char *, char *);
int rc_import_env(struct env *, char **);

/*	external declarations	*/

extern char *conf_authserver[SERVER_MAX];
extern char *conf_acctserver[SERVER_MAX];
extern int  conf_authserver_port[SERVER_MAX];
extern int  conf_acctserver_port[SERVER_MAX];
extern char *conf_clients;
extern char *conf_dictionary;
extern char *conf_default_realm;
extern int conf_auth_type;
extern int conf_radius_timeout;
extern int conf_radius_retries;
extern char *conf_login_local;
extern char *conf_login_radius;
extern int conf_login_tries;
extern char *conf_seqfile;
extern int conf_login_timeout;
extern char *conf_mapfile;

extern int conf_numauth;	/* number of authentication servers */
extern int conf_numacct;	/* number of accounting servers */

#endif /* RADIUS_H */
